import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { memo, useState, useMemo, useCallback } from 'react';
import classNames from 'classnames';
import { mapValues } from 'lodash';
import { getChangeKey, getChangeLineNum, needBeNewFile } from '../../utils';
import CodeCell from '../CodeCell';
import { composeCallback, renderDefaultBy, wrapInAnchorBy } from '../utils';
function useBoundCallbacks(callbacks, arg, hoverOn, hoverOff) {
    return useMemo(() => {
        const output = mapValues(callbacks, fn => (e) => fn && fn(arg, e));
        output.onMouseEnter = composeCallback(hoverOn, output.onMouseEnter);
        output.onMouseLeave = composeCallback(hoverOff, output.onMouseLeave);
        return output;
    }, [callbacks, hoverOn, hoverOff, arg]);
}
function useBoolean() {
    const [value, setValue] = useState(false);
    const on = useCallback(() => setValue(true), []);
    const off = useCallback(() => setValue(false), []);
    return [value, on, off];
}
function renderGutterCell(className, change, changeKey, side, gutterAnchor, anchorTarget, events, inHoverState, renderGutter) {
    const gutterOptions = {
        change,
        side,
        inHoverState,
        renderDefault: renderDefaultBy(change, side),
        wrapInAnchor: wrapInAnchorBy(gutterAnchor, anchorTarget),
    };
    return (_jsx("td", { className: className, ...events, "data-change-key": changeKey, children: renderGutter(gutterOptions) }));
}
function UnifiedChange(props) {
    const { change, selected, tokens, className, gutterClassName, codeClassName, gutterEvents, codeEvents, hideGutter, gutterAnchor, generateAnchorID, renderToken, renderGutter, oldBlameLines, newBlameLines, } = props;
    const { type, content } = change;
    const changeKey = getChangeKey(change);
    const changeNo = getChangeLineNum(change);
    const [hover, hoverOn, hoverOff] = useBoolean();
    const eventArg = useMemo(() => ({ change }), [change]);
    const boundGutterEvents = useBoundCallbacks(gutterEvents, eventArg, hoverOn, hoverOff);
    const boundCodeEvents = useBoundCallbacks(codeEvents, eventArg, hoverOn, hoverOff);
    const [isVisible, setIsVisible] = useState(false);
    const [position, setPosition] = useState({ top: 0, left: 0 });
    const handleMouseEnter = (event) => {
        // 获取鼠标位置
        const { clientX, clientY } = event;
        setPosition({ top: clientY, left: clientX + 10 }); // 在光标右侧显示，增加 10 像素的偏移
        setIsVisible(true);
    };
    const handleMouseLeave = () => {
        setIsVisible(false);
    };
    const anchorID = generateAnchorID(change);
    // 先留着这个
    const blameClassNameValue = classNames('diff-blame', 'diff-gutter', 'diff-gutter-normal');
    const gutterClassNameValue = classNames('diff-gutter', `diff-gutter-${type}`, gutterClassName, { 'diff-gutter-selected': selected });
    const codeClassNameValue = classNames('diff-code', `diff-code-${type}`, codeClassName, { 'diff-code-selected': selected });
    // eslint-disable-next-line no-console
    // console.log('UnifiedChange', oldBlameLines, newBlameLines);
    const oldBlameItem = oldBlameLines?.find(item => changeNo >= item.commit.start_line && changeNo <= item.commit.end_line);
    const newBlameItem = newBlameLines?.find(item => changeNo >= item.commit.start_line && changeNo <= item.commit.end_line);
    const beNew = needBeNewFile(change);
    if (oldBlameLines && oldBlameLines.length > 0 || newBlameLines && newBlameLines.length > 0) {
        // eslint-disable-next-line no-console
        console.log('beNew', changeNo, beNew, oldBlameItem, newBlameItem);
    }
    return (_jsxs("tr", { id: anchorID, className: classNames('diff-line', className), children: [!hideGutter && (oldBlameItem || newBlameItem) ? (_jsxs(_Fragment, { children: [newBlameItem ? (_jsx("td", { onMouseEnter: handleMouseEnter, onMouseLeave: handleMouseLeave, className: blameClassNameValue, children: newBlameItem.commit.committer_name }, `blame_${changeNo}`)) : (_jsx("td", { onMouseEnter: handleMouseEnter, onMouseLeave: handleMouseLeave, className: blameClassNameValue, children: oldBlameItem?.commit?.committer_name }, `blame_${changeNo}`)), isVisible && (_jsx("div", { style: {
                            position: 'absolute',
                            top: position.top,
                            left: position.left,
                            backgroundColor: 'white',
                            border: '1px solid #ccc',
                            padding: '10px',
                            boxShadow: '0 2px 5px rgba(0, 0, 0, 0.15)',
                        }, children: "This is the tooltip content!" }))] })) : null, hideGutter && (_jsx("td", { className: blameClassNameValue }, `blame_${changeNo}_1`)), !hideGutter && renderGutterCell(gutterClassNameValue, change, changeKey, 'old', gutterAnchor, anchorID, boundGutterEvents, hover, renderGutter), !hideGutter && renderGutterCell(gutterClassNameValue, change, changeKey, 'new', gutterAnchor, anchorID, boundGutterEvents, hover, renderGutter), _jsx(CodeCell, { className: codeClassNameValue, changeKey: changeKey, text: content, tokens: tokens, renderToken: renderToken, ...boundCodeEvents })] }));
}
export default memo(UnifiedChange);
//# sourceMappingURL=UnifiedChange.js.map